@@ -10,7 +10,7 @@ import os
 from telethon.tl.types import Message, DocumentAttributeFilename
 from telethon.utils import pack_bot_file_id
 
-from telegram_upload.exceptions import ThumbError, TelegramUploadNoSpaceError
+from telegram_upload.exceptions import ThumbError, TelegramUploadDataLoss, TelegramUploadNoSpaceError
 from telegram_upload.files import get_file_attributes, get_file_thumb
 from telethon.version import __version__ as telethon_version
 from telethon import TelegramClient
@@ -32,12 +32,16 @@ def phone_match(value):
 
 
 def get_progress_bar(action, file, length):
-    bar = click.progressbar(label='{} {}'.format(action, file), length=length)
+    bar = click.progressbar(label='{} "{}"'.format(action, file), length=length)
 
     def progress(current, total):
         bar.pos = 0
         bar.update(current)
-    return progress
+    return progress, bar
+
+
+def truncate(text, max_length):
+    return (text[:max_length - 3] + '...') if len(text) > max_length else text
 
 
 class Client(TelegramClient):
@@ -56,35 +60,41 @@ class Client(TelegramClient):
                              first_name=first_name, last_name=last_name, max_attempts=max_attempts)
 
     def send_files(self, entity, files, delete_on_success=False, print_file_id=False,
-                   force_file=False, forward=(), caption=None):
+                   force_file=False, forward=(), caption=None, no_thumbnail=False):
         for file in files:
-            progress = get_progress_bar('Uploading', os.path.basename(file), os.path.getsize(file))
-            name = '.'.join(os.path.basename(file).split('.')[:-1])
+            file_name = os.path.basename(file)
+            file_size = os.path.getsize(file)
+            progress, bar = get_progress_bar('Uploading', file_name, file_size)
+            name = '.'.join(file_name.split('.')[:-1])
             thumb = None
-            try:
-                thumb = get_file_thumb(file)
-            except ThumbError as e:
-                click.echo('{}'.format(e), err=True)
-            caption = caption[:CAPTION_MAX_LENGTH] if caption else caption
-            caption = caption or ((name[:CAPTION_MAX_LENGTH] + '..') if len(name) > CAPTION_MAX_LENGTH else name)
+            if not no_thumbnail:
+                try:
+                    thumb = get_file_thumb(file)
+                except ThumbError as e:
+                    click.echo('{}'.format(e), err=True)
+            file_caption = truncate(caption if caption is not None else name, CAPTION_MAX_LENGTH)
             try:
                 if force_file:
-                    attributes = [DocumentAttributeFilename(file)]
+                    attributes = [DocumentAttributeFilename(file_name)]
                 else:
                     attributes = get_file_attributes(file)
-                message = self.send_file(entity, file, thumb=thumb,
-                                         caption=caption, force_document=force_file,
-                                         progress_callback=progress, attributes=attributes)
-            except Exception:
-                raise
+                try:
+                    message = self.send_file(entity, file, thumb=thumb,
+                                             caption=file_caption, force_document=force_file,
+                                             progress_callback=progress, attributes=attributes)
+                    if hasattr(message.media, 'document') and file_size != message.media.document.size:
+                        raise TelegramUploadDataLoss(
+                            'Remote document size: {} bytes (local file size: {} bytes)'.format(
+                                message.media.document.size, file_size))
+                finally:
+                    bar.render_finish()
             finally:
                 if thumb:
                     os.remove(thumb)
-            click.echo()
             if print_file_id:
                 click.echo('Uploaded successfully "{}" (file_id {})'.format(file, pack_bot_file_id(message.media)))
             if delete_on_success:
-                click.echo('Deleting {}'.format(file))
+                click.echo('Deleting "{}"'.format(file))
                 os.remove(file)
             self.forward_to(message, forward)
 
@@ -107,11 +117,13 @@ class Client(TelegramClient):
                         filename, sizeof_fmt(message.document.size - free_disk_usage())
                     )
                 )
-            progress = get_progress_bar('Downloading', filename, message.document.size)
-            self.download_media(message, progress_callback=progress)
+            progress, bar = get_progress_bar('Downloading', filename, message.document.size)
+            try:
+                self.download_media(message, progress_callback=progress)
+            finally:
+                bar.render_finish()
             if delete_on_success:
                 self.delete_messages(entity, [message])
-            print()
 
     def forward_to(self, message, destinations):
         for destination in destinations:
